/*
  Add or delete users from a group, or set the maxresults, maxscanrows, maxlocktime, and timeout limits for the members of a group.

*/
function GroupCommand(data) {

    Object.defineProperties(this, {
        /*
          The name of the group, as entered on the command line.

        */
        "group": {
            value: data ? data.group : undefined,
            enumerable: true,
            writable: true
        },
        /*
          The maximum number of results that members of this group can access from the service from a single command.
          The default value is `unset`.

        */
        "maxResults": {
            value: data ? data.maxResults : undefined,
            enumerable: true,
            writable: true
        },
        /*
          The maximum number of rows that members of this group can scan from the service from a single command.
          The default value is `unset`.

        */
        "maxScanRows": {
            value: data ? data.maxScanRows : undefined,
            enumerable: true,
            writable: true
        },
        /*
          The maximum length of time (in milliseconds) that any one operation can lock any database table when scanning data.
          The default value is `unset`.

        */
        "maxLockTime": {
            value: data ? data.maxLockTime : undefined,
            enumerable: true,
            writable: true
        },
        /*
          The maximum number of files that a member of a group can open using a single command.

        */
        "maxOpenFiles": {
            value: data ? data.maxOpenFiles : undefined,
            enumerable: true,
            writable: true
        },
        /*
          The duration (in seconds) of the validity of a session ticket created by p4 login.
          The default value is 43,200 seconds (12 hours).
          To create a ticket that does not expire, set the Timeout: field to `unlimited`.

        */
        "timeout": {
            value: data ? data.timeout : undefined,
            enumerable: true,
            writable: true
        },
        /*
          The length of time (in seconds) for which passwords for users in this group remain valid.
          To disable password aging, use a value of unset.

        */
        "passwordTimeout": {
            value: data ? data.passwordTimeout : undefined,
            enumerable: true,
            writable: true
        },
        /*
          The LDAP configuration to use when populating the group’s user list from an LDAP query.

        */
        "ldapConfig": {
            value: data ? data.ldapConfig : undefined,
            enumerable: true,
            writable: true
        },
        /*
          The LDAP query used to identify the members of the group.

        */
        "ldapSearchQuery": {
            value: data ? data.ldapSearchQuery : undefined,
            enumerable: true,
            writable: true
        },
        /*
          The LDAP attribute that represents the user’s username.

        */
        "ldapUserAttribute": {
            value: data ? data.ldapUserAttribute : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Names of other Perforce groups.

          To add all users in a previously defined group to the group you’re presently working with, include the group name in the Subgroups: field of the p4 group form.
          Note that user and group names occupy separate namespaces, and thus, groups and users can have the same names.

          Every member of any previously defined group you list in the Subgroups: field will be a member of the group you’re now defining.

        */
        "subgroups": {
            value: data ? data.subgroups : undefined,
            enumerable: true,
            writable: true
        },
        /*
          	Names of other Perforce users.

            Group owners without super access are permitted to administer this group, provided that they use the -a option.

            Group owners are not necessarily members of a group; if a group owner is to be a member of the group, the userid must also be added to the Users: field.

            The specified owner does not have to be a Perforce user.
            You might want to use an arbitrary name if the user does not yet exist, or if you have deleted the user and need a placeholder until you can assign the spec to a new user.

        */
        "owners": {
            value: data ? data.owners : undefined,
            enumerable: true,
            writable: true
        },
        /*
          The Perforce usernames of the group members.

        */
        "users": {
            value: data ? data.users : undefined,
            enumerable: true,
            writable: true
        }
    });

}

module.exports = GroupCommand;
