require 'date'

module HelixWebServices
  # Defines a unique Git Fusion branch.
  class GitFusionRepoBranchConfig
    # Alphanumeric ID for the git branch.\n_Do not change this value once this repo has been cloned_.
    attr_accessor :git_branch_id

    # Defines a name specified in a local repo for a Git branch.\n\nA valid Git branch name.\nDo not edit this value after you clone the repo.
    attr_accessor :git_branch_name

    # Defines a Perforce workspace view mapping that maps Perforce depot paths (left side) to Git work tree paths (right side).\n\nCorrectly formed mapping syntax; must not include any Perforce stream or spec depots, and all depot paths on the right side must match exactly across all branch definitions.\nYou can add and remove only certain types of Perforce branches from this view after you clone the repo.
    attr_accessor :view

    # Defines a Perforce stream that maps to the Git branch.\n\nProvide a stream name using the syntax //streamdepot/mystream.\nA Git Fusion branch can be defined as a view or a stream but not both.\nIf your branch is defined as stream, it can include only one stream.
    attr_accessor :stream

    # Prohibit git pushes that introduce commits to the branch.
    attr_accessor :read_only

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        
        :'git_branch_id' => :'gitBranchId',
        
        :'git_branch_name' => :'gitBranchName',
        
        :'view' => :'view',
        
        :'stream' => :'stream',
        
        :'read_only' => :'readOnly'
        
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        :'git_branch_id' => :'String',
        :'git_branch_name' => :'String',
        :'view' => :'Array<String>',
        :'stream' => :'String',
        :'read_only' => :'String'
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes[:'gitBranchId']
        self.git_branch_id = attributes[:'gitBranchId']
      end
      
      if attributes[:'gitBranchName']
        self.git_branch_name = attributes[:'gitBranchName']
      end
      
      if attributes[:'view']
        if (value = attributes[:'view']).is_a?(Array)
          self.view = value
        end
      end
      
      if attributes[:'stream']
        self.stream = attributes[:'stream']
      end
      
      if attributes[:'readOnly']
        self.read_only = attributes[:'readOnly']
      end
      
    end

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          git_branch_id == o.git_branch_id &&
          git_branch_name == o.git_branch_name &&
          view == o.view &&
          stream == o.stream &&
          read_only == o.read_only
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [git_branch_id, git_branch_name, view, stream, read_only].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /^(true|t|yes|y|1)$/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      when :Object
        value
      else # model
        _model = HelixWebServices.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end

  end
end
