require 'date'

module HelixWebServices
  # A list of entries that can show the layout how users are associated with the different groups in the system.
  class GroupsCommand
    attr_accessor :user

    attr_accessor :group

    attr_accessor :is_sub_group

    attr_accessor :is_owner

    attr_accessor :is_user

    attr_accessor :max_results

    attr_accessor :max_scan_rows

    attr_accessor :max_lock_time

    attr_accessor :max_open_files

    attr_accessor :timeout

    attr_accessor :pass_timeout

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        
        :'user' => :'user',
        
        :'group' => :'group',
        
        :'is_sub_group' => :'isSubGroup',
        
        :'is_owner' => :'isOwner',
        
        :'is_user' => :'isUser',
        
        :'max_results' => :'maxResults',
        
        :'max_scan_rows' => :'maxScanRows',
        
        :'max_lock_time' => :'maxLockTime',
        
        :'max_open_files' => :'maxOpenFiles',
        
        :'timeout' => :'timeout',
        
        :'pass_timeout' => :'passTimeout'
        
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        :'user' => :'String',
        :'group' => :'String',
        :'is_sub_group' => :'String',
        :'is_owner' => :'String',
        :'is_user' => :'String',
        :'max_results' => :'String',
        :'max_scan_rows' => :'String',
        :'max_lock_time' => :'String',
        :'max_open_files' => :'String',
        :'timeout' => :'String',
        :'pass_timeout' => :'String'
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes[:'user']
        self.user = attributes[:'user']
      end
      
      if attributes[:'group']
        self.group = attributes[:'group']
      end
      
      if attributes[:'isSubGroup']
        self.is_sub_group = attributes[:'isSubGroup']
      end
      
      if attributes[:'isOwner']
        self.is_owner = attributes[:'isOwner']
      end
      
      if attributes[:'isUser']
        self.is_user = attributes[:'isUser']
      end
      
      if attributes[:'maxResults']
        self.max_results = attributes[:'maxResults']
      end
      
      if attributes[:'maxScanRows']
        self.max_scan_rows = attributes[:'maxScanRows']
      end
      
      if attributes[:'maxLockTime']
        self.max_lock_time = attributes[:'maxLockTime']
      end
      
      if attributes[:'maxOpenFiles']
        self.max_open_files = attributes[:'maxOpenFiles']
      end
      
      if attributes[:'timeout']
        self.timeout = attributes[:'timeout']
      end
      
      if attributes[:'passTimeout']
        self.pass_timeout = attributes[:'passTimeout']
      end
      
    end

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          user == o.user &&
          group == o.group &&
          is_sub_group == o.is_sub_group &&
          is_owner == o.is_owner &&
          is_user == o.is_user &&
          max_results == o.max_results &&
          max_scan_rows == o.max_scan_rows &&
          max_lock_time == o.max_lock_time &&
          max_open_files == o.max_open_files &&
          timeout == o.timeout &&
          pass_timeout == o.pass_timeout
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [user, group, is_sub_group, is_owner, is_user, max_results, max_scan_rows, max_lock_time, max_open_files, timeout, pass_timeout].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /^(true|t|yes|y|1)$/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      when :Object
        value
      else # model
        _model = HelixWebServices.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end

  end
end
