require 'date'

module HelixWebServices
  # A label specification.\n\nLabels can be either automatic or static.\nAutomatic labels refer to the revisions provided in the View: and Revision: fields.\nStatic labels refer only to those specific revisions tagged by the label by means of either the p4 labelsync or p4 tag commands.
  class LabelCommand
    # The label name.
    attr_accessor :label

    # The label’s owner.\nBy default, the user who created the label.\nOnly the owner of a label can update which files are tagged with the label.\n\nThe specified owner does not have to be a Perforce user.\nYou might want to use an arbitrary name if the user does not yet exist, or if you have deleted the user and need a placeholder until you can assign the spec to a new user.
    attr_accessor :owner

    # The date the label specification was last modified.
    attr_accessor :update

    # The date and time the label was last accessed, either by running p4 labelsync on the label, or by otherwise referring to a file with the label revision specifier @label.\n(Note: Reloading a label with p4 reload does not affect the access time.)
    attr_accessor :access

    # An optional description of the label’s purpose.
    attr_accessor :description

    # Options to control behavior and storage location of labels.\n\n- locked or unlocked: If the label is locked, the list of files tagged with the label cannot be changed with p4 labelsync.\n\n- autoreload or noautoreload. For static labels, if noautoreload is set, the label is stored in db.label, and if autoreload is set, it is stored in the unload depot. This option is ignored for automatic labels. Storing labels in the unload depot can improve performance on sites that make extremely heavy use of labels.
    attr_accessor :options

    # An optional revision specification for an automatic label.\n\nIf you use the # character to specify a revision number, you must use quotes around it in order to ensure that the # is parsed as a revision specifier, and not as a comment field in the form.
    attr_accessor :revision

    # A list of depot files that can be tagged with this label.\nNo files are actually tagged until `p4 labelsync` is invoked.\n\nUnlike client views or branch views, which map one set of files to another, label views consist of a simple list of depot files.
    attr_accessor :view

    # If set, restricts usage of the label to the named server.\nIf unset, this label may be used on any server.
    attr_accessor :server_id

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        
        :'label' => :'label',
        
        :'owner' => :'owner',
        
        :'update' => :'update',
        
        :'access' => :'access',
        
        :'description' => :'description',
        
        :'options' => :'options',
        
        :'revision' => :'revision',
        
        :'view' => :'view',
        
        :'server_id' => :'serverID'
        
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        :'label' => :'String',
        :'owner' => :'String',
        :'update' => :'DateTime',
        :'access' => :'DateTime',
        :'description' => :'String',
        :'options' => :'String',
        :'revision' => :'String',
        :'view' => :'Array<String>',
        :'server_id' => :'String'
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes[:'label']
        self.label = attributes[:'label']
      end
      
      if attributes[:'owner']
        self.owner = attributes[:'owner']
      end
      
      if attributes[:'update']
        self.update = attributes[:'update']
      end
      
      if attributes[:'access']
        self.access = attributes[:'access']
      end
      
      if attributes[:'description']
        self.description = attributes[:'description']
      end
      
      if attributes[:'options']
        self.options = attributes[:'options']
      end
      
      if attributes[:'revision']
        self.revision = attributes[:'revision']
      end
      
      if attributes[:'view']
        if (value = attributes[:'view']).is_a?(Array)
          self.view = value
        end
      end
      
      if attributes[:'serverID']
        self.server_id = attributes[:'serverID']
      end
      
    end

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          label == o.label &&
          owner == o.owner &&
          update == o.update &&
          access == o.access &&
          description == o.description &&
          options == o.options &&
          revision == o.revision &&
          view == o.view &&
          server_id == o.server_id
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [label, owner, update, access, description, options, revision, view, server_id].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /^(true|t|yes|y|1)$/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      when :Object
        value
      else # model
        _model = HelixWebServices.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end

  end
end
